
module cpu (clk, rst, instr, regfile_q);
    parameter NUM_REGS = 4;
    parameter NUM_BITS = 8;
    parameter LOG_NUM_REGS = 2;

    input clk;
    input rst;
    input [8:0] instr;
    
    output reg [NUM_BITS * NUM_REGS - 1: 0] regfile_q;
    reg [NUM_BITS * NUM_REGS - 1: 0] regfile_d;
    
    localparam [2:0] INSTR_ADD = 3'b000;
    localparam [2:0] INSTR_SUB = 3'b001;

    // 000: add
    // 001: sub
    // TODO: You can try and implement these yourself :)
    // 010 xor
    // 011 or
    // 100 and
    // 101 not
    // 110 rot1l
    // 111 rot1r

    // [3 opcode] [2 dest] [2 src1] [2 src2]

    wire [2:0] opcode;
    wire [1:0] dest;
    wire [1:0] src1;
    wire [1:0] src2;
    
    assign opcode = instr[8:6];
    assign dest = instr[5:4];
    assign src1 = instr[3:2];
    assign src2 = instr[1:0];
    
    initial regfile_q = 0;

    always @(posedge clk) begin
        regfile_q <= regfile_d;
    end

    reg [NUM_BITS-1:0] src1_val;
    reg [NUM_BITS-1:0] src2_val;
    reg [NUM_BITS-1:0] res_val;
    
    always @(*) begin
        regfile_d = regfile_q;

        src1_val = regfile_q[(src1 + 1) * NUM_BITS - 1 : src1 * NUM_BITS];
        src2_val = regfile_q[(src2 + 1) * NUM_BITS - 1 : src2 * NUM_BITS];
        case (opcode)
            INSTR_ADD: res_val = src1_val + src2_val;
            INSTR_SUB: res_val = src1_val - src2_val;
            default: res_val = 8'd0;
        endcase

        regfile_d[(dest + 1) * NUM_BITS - 1 : dest * NUM_BITS] = res_val;
    end

endmodule
